@@ -1,4 +1,4 @@
-(* 
+(*
  * Top - An interpreted preambule for the toplevel
  * Copyright (C) 2009 David Rajchenbach-Teller, LIFO, Universite d'Orleans
  *
@@ -31,14 +31,11 @@
    This file is loaded by the magic line in the ocamlinit file.
 *)
 
-(* Set the below to false to disable use of syntax extensions in toplevel *)
-let ext_syntax = true;;
-
 
 (* END CONFIGURATION *)
 
 (* MUST BE ALREADY HANDLED BY .ocamlinit
-#use "topfind";; 
+#use "topfind";;
 *)
 #thread;;
 #require "batteries";;
@@ -46,37 +43,39 @@ let ext_syntax = true;;
 
 if !Sys.interactive then (*Only initialize help and display welcome if we're in interactive mode.*)
 begin
-  Batteries_help.init ();
-  print_endline "      _________________________";
-  print_endline "    [| +   | |   Batteries   - |";
-  print_endline "     |_____|_|_________________|";
-  print_endline "      _________________________";
-  print_endline "     | -  Type '#help;;' | | + |]";
-  print_endline "     |___________________|_|___|";
+  BatteriesHelp.init ();
+  let ver = BatteriesConfig.version in
+  let vlen = String.length ver in
+  let pad = String.make vlen '_' in
+  let pad2 = String.make vlen ' ' in
+  print_endline ("      ___________________"^ pad  ^"_______");
+  print_endline ("    [| +   | |   Batteries " ^ ver ^ "  - |");
+  print_endline ("     |_____|_|___________"^ pad  ^"______|");
+  print_endline ("      ___________________"^ pad  ^"_______");
+  print_endline ("     | -  Type '#help;;' "^ pad2 ^"| | + |]");
+  print_endline ("     |___________________"^ pad  ^"|_|___|");
   print_newline ();
   print_newline ();
   flush_all ()
 end;;
 
 open Batteries;;
-#install_printer Batteries_print.print_uchar;;
-#install_printer Batteries_print.print_ustring;;
-#install_printer Batteries_print.print_rope;;
-#install_printer Batteries_print.print_string_cap_rw;;
-#install_printer Batteries_print.print_string_cap_ro;;
-#install_printer Batteries_print.string_dynarray;;
-#install_printer Batteries_print.int_dynarray;;
-#install_printer Batteries_print.char_dynarray;;
-#install_printer Batteries_print.float_dynarray;;
-
-
-if ext_syntax then begin
-  if !Sys.interactive then 
-    print_endline "Loading syntax extensions...";
-  Topfind.standard_syntax();
-  Topfind.load_deeply ["camlp4"; "batteries.pa_string.syntax"; 
-   		       "batteries.pa_comprehension.syntax"];
-end else 
-  if !Sys.interactive then 
-    print_endline "Batteries Syntax extensions disabled.";
-;;
+#install_printer BatteriesPrint.print_uchar;;
+#install_printer BatteriesPrint.print_ustring;;
+#install_printer BatteriesPrint.print_rope;;
+#install_printer BatteriesPrint.print_string_cap_rw;;
+#install_printer BatteriesPrint.print_string_cap_ro;;
+#install_printer BatteriesPrint.string_dynarray;;
+#install_printer BatteriesPrint.int_dynarray;;
+#install_printer BatteriesPrint.char_dynarray;;
+#install_printer BatteriesPrint.float_dynarray;;
+#install_printer BatteriesPrint.int_set;;
+#install_printer BatteriesPrint.string_set;;
+#install_printer BatteriesPrint.int_pset;;
+#install_printer BatteriesPrint.string_pset;;
+#install_printer BatteriesPrint.rope_pset;;
+#install_printer BatteriesPrint.char_pset;;
+#install_printer BatteriesPrint.int_enum;;
+#install_printer BatteriesPrint.string_enum;;
+#install_printer BatteriesPrint.rope_enum;;
+#install_printer BatteriesPrint.char_enum;;
