@@ -77,7 +77,7 @@ let close e =
 let force t =(** Transform [t] into a list *)
   let rec clone enum count =
     let enum = ref !enum
-    and	count = ref !count in
+    and count = ref !count in
     {
       count = (fun () -> !count);
       next  = (fun () ->
@@ -204,7 +204,7 @@ let init n f = (*Experimental fix for init*)
 
 let get t =
   try   Some (t.next())
-  with	No_more_elements -> None
+  with  No_more_elements -> None
 
 let get_exn t = t.next ()
 
@@ -277,8 +277,8 @@ let take n e =
   let f () =
     if !remaining >= 0 then
       let result = e.next () in
-	decr remaining;
-	result
+    decr remaining;
+    result
     else raise No_more_elements
   in let e = make
        ~next: f
@@ -514,6 +514,10 @@ let find f t =
   with
     No_more_elements -> raise Not_found
 
+(*$T
+  find ((=) 5) (1 -- 10) = 5
+  try ignore (find ((=) 11) (1 -- 10) = 5); false with Not_found -> true
+*)
 
 let find_map f t =
   let rec loop () =
@@ -588,6 +592,13 @@ let rec append ta tb =
   );
   t
 
+(*$T
+  append (List.enum [1;2;3]) (List.enum [4;5]) |> List.of_enum = [1;2;3;4;5]
+  append (List.enum [1;2;3]) (List.enum [4;5]) |> \
+    mapi (Tuple2.curry identity) |> List.of_enum = [0,1;1,2;2,3;3,4;4,5]
+
+*)
+
 let prefix_action f t =
   let full_action e =
     e.count <- (fun () -> t.count());
@@ -611,7 +622,7 @@ let suffix_action_without_raise (f:unit -> 'a) (t:'a t) =
       try  t.next ()
       with No_more_elements -> f() );
     clone = (fun () -> t.clone());  (* needs to be delayed because [t] may
-								       mutate and we want the newest clone
+                                       mutate and we want the newest clone
                                        function *)
     fast  = t.fast
   }
@@ -631,7 +642,8 @@ let rec concat t =
   from2 next clone
 
 (*$T concat
-  let e = List.enum [ [| 1; 2; 3; 4|]; [| 5; 6 |] ] |> map Array.enum |> concat in drop 1 e; (count e) = (count (clone e))
+  let e = List.enum [ [| 1; 2; 3; 4|]; [| 5; 6 |] ] |> map Array.enum \
+    |> concat in drop 1 e; (count e) = (count (clone e))
 *)
 
 
@@ -644,7 +656,7 @@ let switchn n f e =
     let my_queue  = queues.(i) in
     if Queue.is_empty my_queue then (*Need to fetch next*)
       let rec aux () =     (*Keep fetching until an appropriate
-							     item has been found*)
+                                 item has been found*)
         let next_item = e.next()    in
         let position  = f next_item in
         if  i = position then next_item
@@ -661,6 +673,11 @@ let switch f e =
   let a = switchn 2 (fun x -> if f x then 0 else 1) e in
   (a.(0), a.(1))
 
+(*$T
+  List.enum [1;2;3;4] |> switch (fun x -> x mod 2 = 0) |> \
+    Tuple2.mapn List.of_enum = ([2;4], [1;3])
+*)
+
 let partition = switch
 
 (*$T partition
@@ -695,6 +712,11 @@ let repeat ?times x = match times with
   | Some n ->
     init n (fun _ -> x)
 
+(*$T
+  repeat ~times:5 0 |> List.of_enum = [0;0;0;0;0]
+  repeat 1 |> take 3 |> List.of_enum = [1;1;1]
+*)
+
 let cycle ?times x =
   let enum   =
     match times with
@@ -702,12 +724,20 @@ let cycle ?times x =
     | Some n -> init n (fun _ -> clone x)
   in concat enum
 
+(*$T
+  cycle ~times:5 (singleton 1) |> List.of_enum = [1;1;1;1;1]
+  cycle (List.enum [1;2]) |> take 5 |> List.of_enum = [1;2;1;2;1]
+*)
+
 let range ?until x =
   let cond =  match until with
     | None   -> ( fun _ -> true   )
     | Some n -> ( fun m -> m <= n )
   in seq x ( ( + ) 1 ) cond
 
+(*$T
+  range 1 ~until:5 |> List.of_enum = [1;2;3;4;5]
+*)
 
 let drop n e =
   for i = 1 to n do
@@ -731,7 +761,7 @@ let drop_while p e =
       print_string "filtering\n";
       if p x then (aux ())
       else (push e x;
-	    raise No_more_elements)
+        raise No_more_elements)
   in
     append (from aux) e*)
 
@@ -777,29 +807,72 @@ let span f t =
   in
   (from head, from tail)
 
+(*$T span
+  List.enum [1;2;3;4;5] |> span (fun x-> x<4) |> Tuple2.mapn List.of_enum = \
+    ([1;2;3], [4;5])
+  *)
+
+(*$Q
+  (Q.list Q.small_int) (fun l -> \
+    let avg = List.fold_left (+) 0 l / (max 1 @@ List.length l) in \
+    let l' = List.sort Int.compare l in \
+    let f x = x < avg in \
+    Tuple2.mapn List.of_enum (span f @@ List.enum l' ) = \
+    (List.of_enum @@ take_while f @@ List.enum l', \
+     List.of_enum @@ drop_while f @@ List.enum l'))
+*)
+
 let while_do cont f e =
   let (head, tail) = span cont e in
   append (f head) tail
 
 let break test e = span (fun x -> not (test x)) e
 
-let uniq e =
+let uniq_by cmp e =
   match peek e with
     None -> empty ()
   | Some first ->
     let prev = ref first in
-    let not_last x = (BatRef.post prev (fun _ -> x)) != x in
+    let not_last x = not (cmp (BatRef.post prev (fun _ -> x)) x) in
     let result = filter not_last e in
     push result first;
     result
 
+let uniq e =
+  uniq_by (=) e
+
+let uniqq e =
+  uniq_by (==) e
+
+
+(*$T
+  List.enum [1;1;2;3;3;2] |> uniq |> List.of_enum = [1;2;3;2]
+  List.enum [1;1;2;3;3;2] |> uniqq |> List.of_enum = [1;2;3;2]
+  List.enum ["a";"a";"b";"c";"c";"b"] |> uniq |> List.of_enum = ["a";"b";"c";"b"]
+  List.enum ["a";"A";"b";"c";"C";"b"] \
+    |> uniq_by (fun a b -> String.lowercase a = String.lowercase b) \
+    |> List.of_enum = ["a";"b";"c";"b"]
+*)
+
 let dup t      = (t, t.clone())
 
+(*$Q
+  (Q.list Q.small_int) (fun l -> \
+    List.enum l |> dup |> Tuple2.mapn List.of_enum |> Tuple2.uncurry (=))
+*)
+
+let min_count x y =
+  let count x = try Some (x.count ()) with Infinite_enum -> None in
+  match count x, count y with
+    | None, None -> raise Infinite_enum
+    | Some c, None | None, Some c -> c
+    | Some c1, Some c2 -> min c1 c2
+
 let combine (x,y) =
   if x.fast && y.fast then (*Optimized case*)
     let rec aux (x,y) =
       {
-        count = (fun () -> min (x.count ()) (y.count ())) ;
+        count = (fun () -> min_count x y)                 ;
         next  = (fun () -> (x.next(), y.next()))          ;
         clone = (fun () -> aux (x.clone(), y.clone()))    ;
         fast  = true
@@ -807,6 +880,15 @@ let combine (x,y) =
     in aux (x,y)
   else from (fun () -> (x.next(), y.next()))
 
+(*$T
+  combine (List.enum [1;2;3], List.enum ["a";"b"]) \
+    |> List.of_enum = [1, "a"; 2, "b"]
+  combine (List.enum [1;2;3], repeat "a") \
+    |> List.of_enum = [1,"a"; 2,"a"; 3,"a"]
+  combine (List.enum [1;2;3], repeat "a") \
+    |> Enum.count = 3
+*)
+
 let uncombine e =
   let advance    = ref `first
   and queue_snd  = Queue.create ()
@@ -903,6 +985,7 @@ let clump clump_size add get e = (* convert a uchar enum into a ustring enum *)
     | None   -> raise No_more_elements
     | Some x ->
       add x;
+      junk e; (* don't get [x] twice *)
       (try
         for i = 2 to clump_size do
           add (e.next ())
@@ -912,6 +995,15 @@ let clump clump_size add get e = (* convert a uchar enum into a ustring enum *)
   in
   from next
 
+(*$T clump
+  let l = RefList.empty() in \
+  Char.range 'a' ~until:'k' |> \
+  clump 4 (RefList.push l) \
+    (fun()-> String.implode \
+      (RefList.to_list l |> tap (fun _ -> RefList.clear l) |> List.rev)) \
+  |> List.of_enum = ["abcd"; "efgh"; "ijk"]
+*)
+
 (* mutable state used for {!cartesian_product}. Use a module to have a private namespace. *)
 module ProductState = struct
   type ('a, 'b) current_state =
@@ -1025,10 +1117,15 @@ let cartesian_product e1 e2 =
 (*$Q cartesian_product
   Q.(pair (list small_int) (list small_int)) \
     (fun (l1,l2) -> \
+      let l1 = List.take 5 l1 in \
+      let l2 = List.take 4 l2 in \
       cartesian_product (List.enum l1) (List.enum l2) |> count = \
       List.length l1 * List.length l2)
   Q.(pair (list small_int) (list small_int)) \
-    (fun (l1,l2) -> cartesian_product (List.enum l1) (List.enum l2) \
+    (fun (l1,l2) -> \
+      let l1 = List.take 5 l1 in \
+      let l2 = List.take 4 l2 in \
+      cartesian_product (List.enum l1) (List.enum l2) \
       |> List.of_enum |> List.length = List.length l1 * List.length l2)
 *)
 
@@ -1070,9 +1167,10 @@ let arg_max f enum =
 (*$T arg_max
    List.enum ["cat"; "canary"; "dog"; "dodo"; "ant"; "cow"] \
       |> arg_max String.length = "canary"
-*) (*$T arg_min
-     -5 -- 5 |> arg_min (fun x -> x * x + 6 * x - 5) = -3
-   *)
+*)
+(*$T arg_min
+   -5 -- 5 |> arg_min (fun x -> x * x + 6 * x - 5) = -3
+ *)
 
 module Infix = struct
   let ( -- ) x y = range x ~until:y
@@ -1139,6 +1237,11 @@ let merge test a b =
       n
     in append_from (append_from (from aux) a) b
 
+(*$T
+  let a=List.enum [1;3;5] and b = List.enum[2;4] in \
+  let test = let r = ref false in (fun _ _ -> r:= not !r; !r) in \
+  merge test a b |> List.of_enum = [1;2;3;4;5]
+*)
 
 (*let mergen test a =
   ArrayLabels.fold_left ~init:[]
@@ -1150,6 +1253,49 @@ let merge test a b =
     if *)
 
 
+let interleave enums =
+  let enums_len = Array.length enums in
+  if not (enums_len > 0) then empty () else begin
+    let available = Array.make enums_len true
+    and next_idx = Array.init enums_len ((+) 1) in
+    next_idx.((Array.length next_idx) - 1) <- 0 ;
+    let rec next_elem idx =
+      match get enums.(idx) with
+       | Some x -> x , next_idx.(idx)
+       | None -> begin
+           available.(idx) <- false ;
+           let rec loop k =
+             let l = next_idx.(k) in
+             if l = idx then raise No_more_elements else
+             if available.(l) then (next_idx.(idx) <- l ; next_elem l) else loop l
+           in loop idx
+         end
+    in
+    from_loop 0 next_elem
+  end
+
+(*$T interleave
+  let e1 = List.enum [ 8 ; 2 ; 5 ; 2 ] and e2 = List.enum [ -5 ; -7 ; -6 ; 2 ; 1 ; -9 ; 2 ] in \
+  let e = interleave [| e1 ; e2 |] in \
+  List.of_enum e = [ 8 ; -5 ; 2 ; -7 ; 5 ; -6 ; 2 ; 2 ; 1 ; -9 ; 2 ]
+*)
+
+(*$R interleave
+  let e1 = Enum.empty ()
+  and e2 = List.enum [ 8 ; 2 ; 5 ; 2 ]
+  and e3 = List.enum [ -5 ; -7 ; -6 ; 2 ; 1 ; -9 ; 2 ] in
+  let e = interleave [| e1; e2 ; e3 |] in
+  assert_equal (List.of_enum e) [ 8 ; -5 ; 2 ; -7 ; 5 ; -6 ; 2 ; 2 ; 1 ; -9 ; 2 ]
+*)
+
+(*$R interleave
+  let e1 = Enum.empty ()
+  and e2 = Enum.empty ()
+  and e3 = Enum.empty () in
+  let e = interleave [| e1; e2 ; e3 |] in
+  assert_equal (List.of_enum e) [ ]
+*)
+
 let slazy f =
   let constructor = lazy (f ()) in
   make ~next: (fun () -> (Lazy.force constructor).next ())
@@ -1237,6 +1383,11 @@ let equal eq t u =
     | _ -> false
   in aux ()
 
+(*$Q
+  (Q.list Q.small_int) (fun l -> \
+    let e = List.enum l in equal Int.equal e (clone e))
+*)
+
 let rec to_object t =
   object
     method next = t.next ()
@@ -1251,7 +1402,18 @@ let rec of_object o =
 
 let flatten = concat
 
-let concat_map f e = concat (map f e)
+(*$T
+  flatten (map singleton @@ List.enum [1;2;3]) |> List.of_enum = [1;2;3]
+*)
+
+let rec concat_map f t =
+  let tn = ref (empty ()) in
+  let rec next () =
+    try (!tn).next ()
+    with No_more_elements -> tn := f (t.next()); next()
+  in
+  let clone () = append ((!tn).clone()) (concat_map f (t.clone())) in
+  from2 next clone
 
 (*$T concat_map
   (1 -- 10 |> concat_map (fun x -> List.enum [x;-x]) |> sum) = 0
@@ -1300,6 +1462,7 @@ module Labels = struct
   let seq ~init ~f ~cnd  = seq init f cnd
   let unfold ~init ~f = unfold init f
   let compare ?(cmp=Pervasives.compare) t u = compare cmp t u
+  let uniq ?(cmp=(=)) x = uniq_by cmp x
   module LExceptionless = struct
     include Exceptionless
     let find ~f e = find f e
@@ -1344,7 +1507,7 @@ struct
   let fold_monad f init enum =
     let (>>=) = Mon.bind and return = Mon.return in
     let rec fold m = match get enum with
-      |	None -> m
+      | None -> m
       | Some x -> m >>= fun acc -> fold (f acc x)
     in
     fold (return init)
@@ -1357,6 +1520,18 @@ struct
   let bind m f = concat (map f m)
 end
 
+(*$T
+  equal (=) (Monad.return 1) (singleton 1)
+  equal (=) (Monad.bind (List.enum [1;2]) (fun x-> List.enum [x+1;x])) \
+    (List.enum [2;1;3;2])
+*)
+
+(*$Q
+  (Q.list Q.small_int) (fun l -> \
+    let id l = Monad.bind l Monad.return in  \
+    List.enum l |> id |> List.of_enum = l)
+*)
+
 module Incubator = struct
   open BatOrd
 
