@@ -207,7 +207,7 @@ let nread i n =
   if n = 0 then
     ""
   else
-    let s = String.create n in
+    let s = Bytes.create n in
     let l = ref n in
     let p = ref 0 in
     try
@@ -261,7 +261,7 @@ let really_nread i n =
   if n < 0 then invalid_arg "BatIO.really_nread";
   if n = 0 then ""
   else
-    let s = String.create n
+    let s = Bytes.create n
     in
     ignore(really_input i s 0 n);
     s
@@ -311,7 +311,7 @@ let read_all i =
   with
     No_more_input
   | Input_closed ->
-    let buf = String.create !pos in
+    let buf = Bytes.create !pos in
     List.iter (fun (s,p) ->
       String.unsafe_blit s 0 buf p (String.length s)
     ) !str;
@@ -342,8 +342,8 @@ let input_string s =
 
 
 let default_buffer_size = 16 (*Arbitrary number. If you replace it, just
-						       don't put something too small, i.e. anything
-						       smaller than 10 is probably a bad idea.*)
+                               don't put something too small, i.e. anything
+                               smaller than 10 is probably a bad idea.*)
 
 let output_string() =
   let b = Buffer.create default_buffer_size in
@@ -389,12 +389,12 @@ let output_channel ?(cleanup=false) ch =
     ~output:(fun s p l -> Pervasives.output ch s p l; l)
     ~close: (if cleanup then fun () ->
         begin
-          (*		   Printf.eprintf "Cleaning up\n%!";*)
+          (*           Printf.eprintf "Cleaning up\n%!";*)
           Pervasives.close_out ch
         end
       else fun () ->
         begin
-          (*		   Printf.eprintf "Not cleaning up\n%!";*)
+          (*           Printf.eprintf "Not cleaning up\n%!";*)
           Pervasives.flush ch
         end)
     ~flush: (fun ()    -> Pervasives.flush ch)
