@@ -161,8 +161,8 @@ let mid a b =
 (*$Q mid
   (Q.pair Q.int Q.int) (fun (a,b) -> \
     let m = mid a b in \
-    a <= b ==> (a <= m && m <= b && abs ((m-a) - (b-m)) <= 1) && \
-    b < a ==> (b <= m && m <= a && abs ((m-b) - (a-m)) <= 1))
+    (a <= b && a <= m && m <= b && abs ((m-a) - (b-m)) <= 1) || \
+    (b <  a && b <= m && m <= a && abs ((m-b) - (a-m)) <= 1))
   (Q.int) (fun a -> mid a a = a)
 *)
 
@@ -201,6 +201,20 @@ let popcount_sparse x =
   (Q.int) (fun x -> popcount x = popcount_sparse x)
 *)
 
+let copysign n o = match n with
+  | 0 -> 0
+  | n when n > 0 -> o
+  | _ -> - o
+
+(*$T copysign
+  copysign 2 1 = 1
+  copysign 3 1 = 1
+  copysign 3 5 = 5
+  copysign max_int min_int = min_int
+  copysign (-22) 12 = -12
+  copysign 0 42 = 0
+*)
+
 module BaseSafeInt = struct
   include BaseInt
 
@@ -209,12 +223,12 @@ module BaseSafeInt = struct
 
   let add a b =
     let c = Pervasives.( + ) a b in
-    if a < 0 && b < 0 && c >= 0 || a > 0 && b > 0 && c <= 0	then raise Overflow
+    if a < 0 && b < 0 && c >= 0 || a > 0 && b > 0 && c <= 0 then raise Overflow
     else c
 
   let sub a b =
     let c = Pervasives.( - ) a b in
-    if a < 0 && b > 0 && c >= 0 || a > 0 && b < 0 && c <= 0	then raise Overflow
+    if a < 0 && b > 0 && c >= 0 || a > 0 && b < 0 && c <= 0 then raise Overflow
     else c
 
   let neg x = if x <> min_int then ~- x else raise Overflow
@@ -225,30 +239,30 @@ module BaseSafeInt = struct
 
   let abs x = if x <> min_int then abs x else raise Overflow
 
-  (*This function used to assume that in case of overflow the result would be
-    different when computing in 31 bits (resp 63 bits) and in 32 bits (resp 64
-    bits). This trick turned out to be *wrong* on 64-bit machines, where
-    [Nativeint.mul 2432902008176640000n 21n] and [2432902008176640000 * 21]
-    yield the same result, [-4249290049419214848]. *)
-  let shift_bits, mask =
-    if Sys.word_size = 32 then 16,0xFFFF else 32, (1 lsl 32) - 1
+  (* Performance hack: if both operands of the multiplication operator can be
+     represented using the specified amount of bits (not counting the sign
+     bit), then it is safe to assume that overflow does not happen. *)
+  let mul_shift_bits =
+    match Sys.word_size with
+      | 64 -> 31                (* 64 = sign bit + 31*2 + tag bit *)
+      | 32 -> 15                (* 32 = sign bit + 15*2 + tag bit *)
+      | _  -> 0
 
-  let mul a b =
-    match a asr shift_bits, b asr shift_bits with
-    |	0,0 -> a * b
-    | 0,bh ->
-      let al = a land mask in
-      let cross = bh * al in
-      if cross > mask then raise Overflow;
-      let bl = b land mask in
-      add (cross lsl shift_bits) (al*bl)
-    | ah, 0 ->
-      let bl = b land mask in
-      let cross = ah * bl in
-      if cross > mask then raise Overflow;
-      let al = a land mask in
-      add (cross lsl shift_bits) (al+bl)
-    | _,_ -> raise Overflow
+  let mul (a: int) (b: int) : int =
+    let open Pervasives in
+    if ((abs a) lor (abs b)) asr mul_shift_bits <> 0
+    then begin match (a > 0, b > 0) with
+      | (true, true) when a > (max_int / b) ->
+          raise BatNumber.Overflow
+      | (true, false) when b < (min_int / a) ->
+          raise BatNumber.Overflow
+      | (false, true) when a < (min_int / b) ->
+          raise BatNumber.Overflow
+      | (false, false) when a <> 0 && (b < (max_int / a)) ->
+          raise BatNumber.Overflow
+      | _ -> ()
+    end;
+    a * b
 
   let pow a b =
     if b < 0
@@ -277,6 +291,14 @@ end
   Result.(catch (List.reduce Safe_int.mul) \
     [1 lsl 18 * 21; 3*3*3*3*3*3*3*3; 5*5*5*5*7*7*11*13*17*19] \
       |> is_exn Number.Overflow)
+  Safe_int.mul 0 min_int = 0
+  Safe_int.mul min_int 0 = 0
+  Safe_int.mul 1 min_int = min_int
+  Safe_int.mul min_int 1 = min_int
+  Safe_int.mul (-1) max_int = -max_int
+  Safe_int.mul max_int (-1) = -max_int
+  Result.(catch (Safe_int.mul min_int) (-1) |> is_exn Number.Overflow)
+  Result.(catch (Safe_int.mul (-1)) min_int |> is_exn Number.Overflow)
   Result.(catch (Safe_int.Infix.(+) max_int) 1 |> is_exn Number.Overflow)
 *)
 
