@@ -56,7 +56,7 @@ let input_all ic =
   let rec loop acc total buf ofs =
     let n = input ic buf ofs (buf_len - ofs) in
     if n = 0 then
-      let res = String.create total in
+      let res = Bytes.create total in
       let pos = total - ofs in
       let _ = String.blit buf 0 res pos ofs in
       let coll pos buf =
@@ -69,9 +69,9 @@ let input_all ic =
       let new_ofs = ofs + n in
       let new_total = total + n in
       if new_ofs = buf_len then
-        loop (buf :: acc) new_total (String.create buf_len) 0
+        loop (buf :: acc) new_total (Bytes.create buf_len) 0
       else loop acc new_total buf new_ofs in
-  loop [] 0 (String.create buf_len) 0
+  loop [] 0 (Bytes.create buf_len) 0
 
 let input_file ?(bin=false) fname =
   let ch = (if bin then open_in_bin else open_in) fname in
@@ -132,8 +132,8 @@ let rec dump r =
       "(" ^ String.concat ", " (List.map dump fields) ^ ")"
     | x when x = Obj.lazy_tag ->
       (* Note that [lazy_tag .. forward_tag] are < no_scan_tag.  Not
-		 * clear if very large constructed values could have the same
-		 * tag. XXX *)
+         * clear if very large constructed values could have the same
+         * tag. XXX *)
       opaque "lazy"
     | x when x = Obj.closure_tag ->
       opaque "closure"
@@ -145,7 +145,7 @@ let rec dump r =
         | _ -> assert false
       in
       (* No information on decoding the class (first field).  So just print
-		 * out the ID and the slots. *)
+         * out the ID and the slots. *)
       "Object #" ^ dump id ^ " (" ^ String.concat ", " (List.map dump slots) ^ ")"
     | x when x = Obj.infix_tag ->
       opaque "infix"
@@ -227,14 +227,16 @@ let open_in_bin name  = BatIO.input_channel ~cleanup:true (open_in_bin name)
 let open_in_gen mode perm filename =
   BatIO.input_channel ~cleanup:true (open_in_gen mode perm filename)
 
-let input_char        = BatIO.read
-let input_line ic     = try BatIO.read_line ic with BatIO.No_more_input -> raise End_of_file
-let input             = BatIO.input
+let wrap_inner_io f a = try f a with BatIO.No_more_input -> raise End_of_file
+let input_char        = wrap_inner_io BatIO.read
+let input_line        = wrap_inner_io BatIO.read_line
+let input             = wrap_inner_io BatIO.input
 let really_input inp buf pos len =
-  ignore (BatIO.really_input inp buf pos len)
-let input_byte        = BatIO.read_byte
-let input_binary_int  = BatIO.read_i32
-let input_binary_float inp= BatInt64.float_of_bits (BatIO.read_i64 inp)
+  wrap_inner_io ignore (BatIO.really_input inp buf pos len)
+let input_byte        = wrap_inner_io BatIO.read_byte
+let input_binary_int  = wrap_inner_io BatIO.read_i32
+let input_binary_float inp =
+  wrap_inner_io BatInt64.float_of_bits (BatIO.read_i64 inp)
 let close_in          = BatIO.close_in
 let close_in_noerr inp=
   try BatIO.close_in inp
