@@ -75,6 +75,38 @@ val iter : ('a -> unit) -> 'a t -> unit
     from the least recently entered to the most recently entered.
     The queue itself is unchanged. *)
 
+val map : ('a -> 'b) -> 'a t -> 'b t
+(** [map f q] applies function [f] to each element of the queue
+   and returns a new queue [q'] with the results returned by [f].
+   Order is preserved and [q] is not consumed. So that if
+   [take q] returns [x] [take q'] will return [f x].
+
+   @since 2.3.0
+*)
+
+val filter : ('a -> bool) -> 'a t -> 'a t
+(** [filter p q] returns a new queue that contain the elements of [q]
+    that satisfy the predicate [p], in the same order.
+
+    @since 2.4
+*)
+
+val filter_map : ('a -> 'b option) -> 'a t -> 'b t
+(** [filter_map f q] applies [f] to the elements [a0,a1..an] of [q] in order,
+    and returns the queue of the elements [bi] such that [f ai = Some bi],
+    in the corresponding order.
+
+    @since 2.4
+*)
+
+val filter_inplace : ('a -> bool) -> 'a t -> unit
+(** [filter_inplace p q] removes all the elements of the queue [q] that
+    don't satisfy the predicate [p].  The order of the elements
+    in the queue is preserved.
+
+    @since 2.4
+*)
+
 val fold : ('b -> 'a -> 'b) -> 'b -> 'a t -> 'b
 (** [fold f accu q] is equivalent to [List.fold_left f accu l],
     where [l] is the list of [q]'s elements. The queue remains
