@@ -85,6 +85,30 @@ module Concrete = struct
     | Node(Empty, v, r, _) -> v
     | Node(l, v, r, _) -> min_elt l
 
+  let get_root = function
+    | Empty -> raise Not_found
+    | Node(_l, v, _r, _) -> v
+
+  let pop_min s =
+    let mini = ref (get_root s) in
+    let rec loop = function
+        Empty -> raise Not_found
+      | Node(Empty, v, r, _) -> mini := v; r
+      | Node(l, v, r, _) -> bal (loop l) v r
+    in
+    let others = loop s in
+    (!mini, others)
+
+  let pop_max s =
+    let maxi = ref (get_root s) in
+    let rec loop = function
+        Empty -> raise Not_found
+      | Node(l, v, Empty, _) -> maxi := v; l
+      | Node(l, v, r, _) -> bal l v (loop r)
+    in
+    let others = loop s in
+    (!maxi, others)
+
   let rec max_elt = function
       Empty -> raise Not_found
     | Node(l, v, Empty, _) -> v
@@ -95,7 +119,7 @@ module Concrete = struct
       Empty -> invalid_arg "Set.remove_min_elt"
     | Node(Empty, v, r, _) -> r
     | Node(l, v, r, _) -> bal (remove_min_elt l) v r
-
+    
   (* Merge two trees l and r into one.
      All elements of l must precede the elements of r.
      Assume | height l - height r | <= 2. *)
@@ -126,6 +150,23 @@ module Concrete = struct
       if c = 0 then merge l r else
       if c < 0 then bal (remove cmp x l) v r else bal l v (remove cmp x r)
 
+  let update cmp x y s =
+    if cmp x y <> 0 then
+      add cmp y (remove cmp x s)
+    else
+      let rec loop = function
+        | Empty -> raise Not_found
+        | Node(l, v, r, h) ->
+          let c = cmp x v in
+          if c = 0 then
+            Node(l, y, r, h)
+          else if c < 0 then
+            Node(loop l, v, r, h)
+          else
+            Node(l, v, loop r, h)
+      in
+      loop s
+
   let rec mem cmp x = function
       Empty -> false
     | Node(l, v, r, _) ->
@@ -142,11 +183,33 @@ module Concrete = struct
       Empty -> ()
     | Node(l, v, r, _) -> iter f l; f v; iter f r
 
+  let get_root = function
+    | Empty -> raise Not_found
+    | Node(l, v, r, _) -> v
+
   let rec fold f s accu =
     match s with
       Empty -> accu
     | Node(l, v, r, _) -> fold f r (f v (fold f l accu))
 
+  exception Found
+
+  let at_rank_exn i s =
+    if i < 0 then invalid_arg "Set.at_rank_exn: negative index not allowed";
+    let res = ref (get_root s) in (* raises Not_found if empty *)
+    try
+      let (_: int) =
+        fold (fun node j ->
+            if j <> i then j + 1
+            else begin
+              res := node;
+              raise Found
+            end
+          ) s 0
+      in
+      invalid_arg "Set.at_rank_exn i s: i >= (Set.cardinal s)"
+    with Found -> !res
+
   let map cmp f s =
     fold (fun v acc -> add cmp (f v) acc) s empty
 
@@ -268,6 +331,11 @@ module Concrete = struct
   let elements s = elements_aux [] s
   let to_list = elements
 
+  let to_array s =
+    let acc = BatDynArray.create () in
+    iter (BatDynArray.add acc) s;
+    BatDynArray.to_array acc
+
   let rec cons_iter s t = match s with
       Empty -> t
     | Node (l, e, r, _) -> cons_iter l (C (e, r, t))
@@ -307,6 +375,10 @@ module Concrete = struct
   let of_enum cmp e =
     BatEnum.fold (fun acc elem -> add cmp elem acc) empty e
 
+  let of_list cmp l = List.fold_left (fun a x -> add cmp x a) empty l
+
+  let of_array cmp l = Array.fold_left (fun a x -> add cmp x a) empty l
+
   let print ?(first="{") ?(last="}") ?(sep=",") print_elt out t =
     BatEnum.print ~first ~last ~sep (fun out e -> BatPrintf.fprintf out "%a" print_elt e) out (enum t)
 
@@ -450,6 +522,7 @@ sig
   val find: elt -> t -> elt
   val add: elt -> t -> t
   val remove: elt -> t -> t
+  val update: elt -> elt -> t -> t
   val union: t -> t -> t
   val inter: t -> t -> t
   val diff: t -> t -> t
@@ -460,6 +533,7 @@ sig
   val disjoint: t -> t -> bool
   val compare_subset: t -> t -> int
   val iter: (elt -> unit) -> t -> unit
+  val at_rank_exn: int -> t -> elt
   val map: (elt -> elt) -> t -> t
   val filter: (elt -> bool) -> t -> t
   val filter_map: (elt -> elt option) -> t -> t
@@ -474,16 +548,31 @@ sig
   val cardinal: t -> int
   val elements: t -> elt list
   val to_list: t -> elt list
+  val to_array: t -> elt array
   val min_elt: t -> elt
+  val pop_min: t -> elt * t
+  val pop_max: t -> elt * t
   val max_elt: t -> elt
   val choose: t -> elt
   val pop: t -> elt * t
   val enum: t -> elt BatEnum.t
   val backwards: t -> elt BatEnum.t
   val of_enum: elt BatEnum.t -> t
+  val of_list: elt list -> t
+  val of_array: elt array -> t
   val print :  ?first:string -> ?last:string -> ?sep:string ->
     ('a BatInnerIO.output -> elt -> unit) ->
     'a BatInnerIO.output -> t -> unit
+  module Infix : sig
+    val (<--) : t -> elt -> t (** insertion *)
+    val (<.) : t -> t -> bool  (** strict subset *)
+    val (>.) : t -> t -> bool  (** strict superset *)
+    val (<=.) : t -> t -> bool (** subset *)
+    val (>=.) : t -> t -> bool (** superset *)
+    val (-.) : t -> t -> t     (** difference *)
+    val (&&.) : t -> t -> t   (** intersection *)
+    val (||.) : t -> t -> t   (** union *)
+  end
   (** Operations on {!Set} without exceptions.*)
   module Exceptionless : sig
     val min_elt: t -> elt option
@@ -502,7 +591,6 @@ sig
     val filter_map: f:(elt -> elt option) -> t -> t
     val partition : f:(elt -> bool) -> t -> t * t
   end
-
 end
 (** Output signature of the functor {!Set.Make}. *)
 
@@ -522,9 +610,12 @@ struct
   let backwards t = Concrete.backwards (impl_of_t t)
 
   let remove e t = t_of_impl (Concrete.remove Ord.compare e (impl_of_t t))
+  let update e1 e2 t =
+    t_of_impl (Concrete.update Ord.compare e1 e2 (impl_of_t t))
   let add e t = t_of_impl (Concrete.add Ord.compare e (impl_of_t t))
 
   let iter f t = Concrete.iter f (impl_of_t t)
+  let at_rank_exn i t = Concrete.at_rank_exn i (impl_of_t t)
   let map f t = t_of_impl (Concrete.map Ord.compare f (impl_of_t t))
   let fold f t acc = Concrete.fold f (impl_of_t t) acc
   let filter f t = t_of_impl (Concrete.filter Ord.compare f (impl_of_t t))
@@ -538,6 +629,13 @@ struct
     (t_of_impl l, t_of_impl r)
 
   let min_elt t = Concrete.min_elt (impl_of_t t)
+  let pop_min t =
+    let mini, others = Concrete.pop_min (impl_of_t t) in
+    (mini, t_of_impl others)
+  let pop_max t =
+    let maxi, others = Concrete.pop_max (impl_of_t t) in
+    (maxi, t_of_impl others)
+
   let max_elt t = Concrete.max_elt (impl_of_t t)
   let choose t = Concrete.choose (impl_of_t t)
   let pop t =
@@ -563,6 +661,7 @@ struct
   let singleton e = t_of_impl (Concrete.singleton e)
   let elements t = Concrete.elements (impl_of_t t)
   let to_list = elements
+  let to_array t = Concrete.to_array (impl_of_t t)
 
   let union s1 s2 =
     t_of_impl (Concrete.union Ord.compare (impl_of_t s1) (impl_of_t s2))
@@ -597,9 +696,23 @@ struct
 
   let compare_subset s1 s2 = compare_subset (impl_of_t s1) s2
 
+  let of_list l = t_of_impl (Concrete.of_list Ord.compare l)
+  let of_array a = t_of_impl (Concrete.of_array Ord.compare a)
+
   let print ?first ?last ?sep print_elt out t =
     Concrete.print ?first ?last ?sep print_elt out (impl_of_t t)
 
+  module Infix = struct
+    let (<--) s x = add x s
+    let (<.) a b = not (equal a b) && subset a b
+    let (>.) a b = not (equal a b) && subset b a
+    let (<=.) = subset
+    let (>=.) a b = subset b a
+    let (-.) = diff
+    let (&&.) = inter
+    let (||.) = union
+  end
+
   module Exceptionless =
   struct
     let min_elt t = try Some (min_elt t) with Not_found -> None
@@ -621,6 +734,32 @@ struct
   end
 end
 
+module Make2(O1 : OrderedType)(O2 : OrderedType) = struct
+  module Set1 = Make(O1)
+  module Set2 = Make(O2)
+  module Product = Make(
+  struct
+    type t = O1.t * O2.t
+    let compare (x1,y1)(x2,y2) =
+      let c = O1.compare x1 x2 in
+        if c = 0 then O2.compare y1 y2 else c
+  end)
+
+  let cartesian_product set1 set2 =
+    let p = Concrete.cartesian_product (Set1.impl_of_t set1) (Set2.impl_of_t set2) in
+    Product.t_of_impl p
+end
+
+(*$T
+  let module S1 = Make(Int) in \
+  let module S2 = Make(String) in \
+  let module P = Make2(Int)(String) in \
+  P.cartesian_product \
+    (List.fold_right S1.add [1;2;3] S1.empty) \
+    (List.fold_right S2.add ["a";"b"] S2.empty) \
+    |> P.Product.to_list = [1, "a"; 1, "b"; 2, "a"; 2, "b"; 3, "a"; 3, "b"]
+*)
+
 module PSet = struct (*$< PSet *)
 
   type 'a t = {
@@ -646,7 +785,9 @@ module PSet = struct (*$< PSet *)
   let find x s = Concrete.find s.cmp x s.set
   let add x s  = { s with set = Concrete.add s.cmp x s.set }
   let remove x s = { s with set = Concrete.remove s.cmp x s.set }
+  let update x y s = { s with set = Concrete.update s.cmp x y s.set }
   let iter f s = Concrete.iter f s.set
+  let at_rank_exn i s = Concrete.at_rank_exn i s.set
   let fold f s acc = Concrete.fold f s.set acc
   let map f s =
     { cmp = Pervasives.compare; set = Concrete.map Pervasives.compare f s.set }
@@ -657,13 +798,22 @@ module PSet = struct (*$< PSet *)
   let cardinal s = fold (fun _ acc -> acc + 1) s 0
   let elements s = Concrete.elements s.set
   let to_list = elements
+  let to_array s = Concrete.to_array s.set
   let choose s = Concrete.choose s.set
   let min_elt s = Concrete.min_elt s.set
+  let pop_min s =
+    let mini, others = Concrete.pop_min s.set in
+    (mini, { s with set = others })
+  let pop_max s =
+    let maxi, others = Concrete.pop_max s.set in
+    (maxi, { s with set = others })
+
   let max_elt s = Concrete.max_elt s.set
   let enum s = Concrete.enum s.set
   let of_enum e = { cmp = compare; set = Concrete.of_enum compare e }
   let of_enum_cmp ~cmp t = { cmp = cmp; set = Concrete.of_enum cmp t }
-  let of_list l = List.fold_left (fun a x -> add x a) empty l
+  let of_list l = { cmp = compare; set = Concrete.of_list compare l }
+  let of_array a = { cmp = compare; set = Concrete.of_array compare a }
   let print ?first ?last ?sep print_elt out s =
     Concrete.print ?first ?last ?sep print_elt out s.set
   let for_all f s = Concrete.for_all f s.set
@@ -697,6 +847,17 @@ module PSet = struct (*$< PSet *)
   let equal s1 s2 = Concrete.equal s1.cmp s1.set s2.set
   let subset s1 s2 = Concrete.subset s1.cmp s1.set s2.set
   let disjoint s1 s2 = Concrete.disjoint s1.cmp s1.set s2.set
+
+  module Infix = struct
+    let (<--) s x = add x s
+    let (<.) a b = not (equal a b) && subset a b
+    let (>.) a b = not (equal a b) && subset b a
+    let (<=.) = subset
+    let (>=.) a b = subset b a
+    let (-.) = diff
+    let (&&.) = intersect
+    let (||.) = union
+  end
 end (*$>*)
 
 type 'a t = 'a Concrete.set
@@ -714,7 +875,7 @@ let mem x s = Concrete.mem Pervasives.compare x s
 
 let find x s = Concrete.find Pervasives.compare x s
 
-(*$T
+(*$T find
   (find 1 (of_list [1;2;3;4;5;6;7;8])) == 1
   (find 8 (of_list [1;2;3;4;5;6;7;8])) == 8
   (find 1 (singleton 1)) == 1
@@ -729,8 +890,22 @@ let add x s  = Concrete.add Pervasives.compare x s
 
 let remove x s = Concrete.remove Pervasives.compare x s
 
+let update x y s = Concrete.update Pervasives.compare x y s
+
 let iter f s = Concrete.iter f s
 
+let at_rank_exn i s = Concrete.at_rank_exn i s
+
+(*$T
+  at_rank_exn 0 (of_list [1;2]) == 1
+  at_rank_exn 1 (of_list [1;2]) == 2
+  try ignore (at_rank_exn 0 empty); false with Not_found -> true
+  try ignore (at_rank_exn (-1) (singleton 1)); false \
+  with Invalid_argument _msg -> true
+  try ignore (at_rank_exn 1 (singleton 1)); false \
+  with Invalid_argument _msg -> true
+*)
+
 let fold f s acc = Concrete.fold f s acc
 
 let map f s = Concrete.map Pervasives.compare f s
@@ -749,6 +924,7 @@ let cardinal s = fold (fun _ acc -> acc + 1) s 0
 
 let elements s = Concrete.elements s
 let to_list = elements
+let to_array s = Concrete.to_array s
 
 let choose s = Concrete.choose s
 
@@ -768,6 +944,25 @@ let min_elt s = Concrete.min_elt s
   )
 *)
 
+let pop_min s = Concrete.pop_min s
+
+(*$T pop_min
+  try ignore (pop_min empty); false with Not_found -> true
+  pop_min (of_list [1;2]) = (1, singleton 2)
+  pop_min (singleton 2) = (2, empty)
+  pop_min (of_list [4;5;6;7]) = (4, of_list [5;6;7])
+*)
+
+let pop_max s = Concrete.pop_max s
+
+(*$T pop_max
+  try ignore (pop_max empty); false with Not_found -> true
+  pop_max (of_list [1;2]) = (2, singleton 1)
+  pop_max (singleton 2) = (2, empty)
+  let maxi, others = pop_max (of_list [4;5;6;7]) in \
+  maxi = 7 && diff others (of_list [4;5;6]) = empty
+*)
+
 let max_elt s = Concrete.max_elt s
 
 let enum s = Concrete.enum s
@@ -776,7 +971,7 @@ let of_enum e = Concrete.of_enum Pervasives.compare e
 
 let backwards s = Concrete.backwards s
 
-let of_list l = List.fold_left (fun a x -> add x a) empty l
+let of_list l = Concrete.of_list Pervasives.compare l
 
 (*$Q of_list
   (Q.list Q.small_int) (fun l -> let xs = List.map (fun i -> i mod 5, i) l in \
@@ -786,6 +981,8 @@ let of_list l = List.fold_left (fun a x -> add x a) empty l
   )
 *)
 
+let of_array a = Concrete.of_array Pervasives.compare a
+
 let print ?first ?last ?sep print_elt out s =
   Concrete.print ?first ?last ?sep print_elt out s
 
@@ -828,8 +1025,36 @@ let disjoint s1 s2 = Concrete.disjoint Pervasives.compare s1 s2
   let s1, s2 = of_list ["a"; "b"; "c"], of_list [1;2;3] in \
     equal (cartesian_product s1 s2) \
           (map BatTuple.Tuple2.swap (cartesian_product s2 s1))
- *)
+*)
 
+(*$inject
+  module TestSet =
+    Set.Make
+      (struct
+        type t = int * int
+        let compare (x, _) (y, _) = BatInt.compare x y
+      end) ;;
+  let ts = TestSet.of_list [(1,0);(2,0);(3,0)] ;;
+*)
+(*$T
+  try ignore(TestSet.update (1, 0) (1, 1) TestSet.empty); false \
+  with Not_found -> true
+  TestSet.update (1,0) (1,1)  ts = TestSet.of_list [(1,1);(2,0);(3,0)]
+  TestSet.update (2,0) (2,1)  ts = TestSet.of_list [(1,0);(2,1);(3,0)]
+  TestSet.update (3,0) (3,1)  ts = TestSet.of_list [(1,0);(2,0);(3,1)]
+  TestSet.update (3,0) (-1,0) ts = TestSet.of_list [(1,0);(2,0);(-1,0)]
+*)
+
+module Infix = struct
+  let (<--) s x = add x s
+  let (<.) a b = not (equal a b) && subset a b
+  let (>.) a b = not (equal a b) && subset b a
+  let (<=.) = subset
+  let (>=.) a b = subset b a
+  let (-.) = diff
+  let (&&.) = intersect
+  let (||.) = union
+end
 
 module Incubator = struct (*$< Incubator *)
   let op_map f s = Concrete.op_map f s
