@@ -32,9 +32,9 @@ let ord = BatOrd.ord String.compare
 
 
 let init len f =
-  let s = create len in
+  let s = Bytes.create len in
   for i = 0 to len - 1 do
-    unsafe_set s i (f i)
+    Bytes.unsafe_set s i (f i)
   done;
   s
 
@@ -364,7 +364,7 @@ let join = concat
 
 let unsafe_slice i j s =
   if i >= j || i = length s then
-    create 0
+    Bytes.create 0
   else
     sub s i (j-i)
 
@@ -491,9 +491,9 @@ let backwards s =
 let of_enum e =
   (* TODO: use a buffer when not fast_count *)
   let l = BatEnum.count e in
-  let s = create l in
+  let s = Bytes.create l in
   let i = ref 0 in
-  BatEnum.iter (fun c -> unsafe_set s (BatRef.post_incr i) c) e;
+  BatEnum.iter (fun c -> Bytes.unsafe_set s (BatRef.post_incr i) c) e;
   s
 (*$T of_enum
     Enum.init 3 (fun i -> char_of_int (i + int_of_char '0')) |> of_enum = "012"
@@ -503,9 +503,9 @@ let of_enum e =
 let of_backwards e =
   (* TODO: use a buffer when not fast_count *)
   let l = BatEnum.count e in
-  let s = create l in
+  let s = Bytes.create l in
   let i = ref (l - 1) in
-  BatEnum.iter (fun c -> unsafe_set s (BatRef.post_decr i) c) e;
+  BatEnum.iter (fun c -> Bytes.unsafe_set s (BatRef.post_decr i) c) e;
   s
 (*$T of_backwards
    "" |> enum |> of_backwards = ""
@@ -515,9 +515,9 @@ let of_backwards e =
 
 let map f s =
   let len = length s in
-  let sc = create len in
+  let sc = Bytes.create len in
   for i = 0 to len - 1 do
-    unsafe_set sc i (f (unsafe_get s i))
+    Bytes.unsafe_set sc i (f (unsafe_get s i))
   done;
   sc
 (*$T map
@@ -526,6 +526,21 @@ let map f s =
    map (String.of_char %> failwith) "" = ""
 *)
 
+let mapi f s =
+  let len = length s in
+  let sc = Bytes.create len in
+  for i = 0 to len - 1 do
+    Bytes.unsafe_set sc i (f i (unsafe_get s i))
+  done;
+  sc
+(*$T mapi
+   mapi (fun _ -> Char.uppercase) "Five" = "FIVE"
+   mapi (fun _ -> Char.uppercase) "" = ""
+   mapi (fun _ -> String.of_char %> failwith) "" = ""
+   mapi (fun i _c -> "0123456789".[9-i]) "0123456789" = "9876543210"
+   ignore (let last = ref (-1) in mapi (fun i _c -> assert (i > !last); last := i; '0') "012345"); true
+*)
+
 let filter_map f s =
   let len = length s in
   let sc = Buffer.create len in
@@ -564,6 +579,17 @@ let fold_left f init str =
    fold_left max 'a' "apples" = 's'
 *)
 
+let fold_lefti f init str =
+  let n = String.length str in
+  let rec loop i result =
+    if i = n then result
+    else loop (i + 1) (f result i str.[i])
+  in loop 0 init
+(*$T fold_lefti
+   fold_lefti (fun a i c->(i,c)::a) [] "foo"=[(2,'o');(1,'o');(0,'f')]
+   fold_lefti (fun a i _->i+a) 0 "" = 0
+*)
+
 
 let fold_right f str init =
   let n = String.length str in
@@ -579,6 +605,19 @@ let fold_right f str init =
    fold_right (fun c a -> if c = ' ' then a+1 else a) "a b c" 0 = 2
 *)
 
+let fold_righti f str init =
+  let n = String.length str in
+  let rec loop i result =
+    if i = 0 then result
+    else
+      let i' = i - 1 in
+      loop i' (f i' str.[i'] result)
+  in loop n init
+(*$T fold_righti
+   fold_righti (fun i c a->(i,c)::a) "foo" []=[(0,'f');(1,'o');(2,'o')]
+   fold_righti (fun i _ a -> a + i) "" 0 = 0
+*)
+
 let iteri f str =
   for i = 0 to (String.length str) - 1 do f i str.[i] done
 
@@ -614,10 +653,10 @@ let to_list = explode
 *)
 
 let implode l =
-  let res = String.create (List.length l) in
+  let res = Bytes.create (List.length l) in
   let rec imp i = function
     | [] -> res
-    | c :: l -> res.[i] <- c; imp (i + 1) l in
+    | c :: l -> Bytes.set res i c; imp (i + 1) l in
   imp 0 l
 (*$T implode
    implode ['b';'a';'r'] = "bar"
@@ -643,7 +682,7 @@ let replace_chars f s =
       loop (i+1) (s :: acc)
   in
   let strs = loop 0 [] in
-  let sbuf = create !tlen in
+  let sbuf = Bytes.create !tlen in
   let pos = ref !tlen in
   let rec loop2 = function
     | [] -> ()
@@ -667,7 +706,7 @@ let replace ~str ~sub ~by =
      let strlen = length str in
      let sublen = length sub in
      let bylen  = length by in
-     let newstr = create (strlen - sublen + bylen) in
+     let newstr = Bytes.create (strlen - sublen + bylen) in
      blit str 0 newstr 0 subpos ;
      blit by 0 newstr subpos bylen ;
      blit str (subpos + sublen) newstr (subpos + bylen) (strlen - subpos - sublen) ;
@@ -691,7 +730,7 @@ let nreplace ~str ~sub ~by =
     | -1 -> idxes, newlen
     | i' -> loop_subst (i'::idxes) (newlen+dlen) i' in
   let idxes, newlen = loop_subst [] strlen strlen in
-  let newstr = create newlen in
+  let newstr = Bytes.create newlen in
   let rec loop_copy i j idxes =
     match idxes with
     | [] ->
@@ -719,7 +758,7 @@ let rev_in_place s =
   let len = String.length s in
   if len > 0 then for k = 0 to (len - 1)/2 do
       let old = s.[k] and mirror = len - 1 - k in
-      s.[k] <- s.[mirror]; s.[mirror] <- old;
+      Bytes.set s k s.[mirror]; Bytes.set s mirror old;
     done
 (*$= rev_in_place as f & ~printer:identity
   (let s="" in f s; s)          ""
@@ -742,9 +781,9 @@ let repeat s n =
 
 let rev s =
   let len = String.length s in
-  let reversed = String.create len in
+  let reversed = Bytes.create len in
   for i = 0 to len - 1 do
-    String.unsafe_set reversed (len - i - 1) (String.unsafe_get s i)
+    Bytes.unsafe_set reversed (len - i - 1) (String.unsafe_get s i)
   done;
   reversed
 
@@ -782,7 +821,7 @@ let splice s1 off len s2 =
   let off = wrap off ~hi:len1 in
   let len = clip ~lo:0 ~hi:(len1 - off) len in
   let out_len = len1 - len + len2 in
-  let s = create out_len in
+  let s = Bytes.create out_len in
   blit s1 0 s 0 off; (* s1 before splice point *)
   blit s2 0 s off len2; (* s2 at splice point *)
   blit s1 (off+len) s (off+len2) (len1 - (off+len)); (* s1 after off+len *)
@@ -997,8 +1036,11 @@ struct
   let to_int        = to_int
   let to_float      = to_float
   let map           = map
+  let mapi          = mapi
   let fold_left     = fold_left
   let fold_right    = fold_right
+  let fold_lefti    = fold_lefti
+  let fold_righti   = fold_righti
   let iter          = iter
   let index         = index
   let rindex        = rindex
@@ -1023,7 +1065,7 @@ struct
   let uncapitalize  = uncapitalize
   let copy          = copy
   let sub           = sub
-  let fill          = fill
+  let fill          = Bytes.fill
   let blit          = blit
   let concat        = concat
   let escaped       = escaped
