@@ -172,6 +172,12 @@ val map : (char -> char) -> string -> string
     is returned.
     @since 4.00.0 *)
 
+val mapi : (int -> char -> char) -> string -> string
+(** [String.mapi f s] calls [f] with each character of [s] and its
+    index (in increasing index order) and stores the results in a new
+    string that is returned.
+    @since 4.02.0 *)
+
 val trim : string -> string
 (** Return a copy of the argument, without leading and trailing
     whitespace.  The characters regarded as whitespace are: [' '],
@@ -370,6 +376,12 @@ val fold_left : ('a -> char -> 'a) -> 'a -> string -> 'a
     [String.fold_left max 'a' "apples" = 's']
 *)
 
+val fold_lefti : ('a -> int -> char -> 'a) -> 'a -> string -> 'a
+(** As [fold_left], but with the index of the element as additional argument
+
+    @since 2.3.0
+*)
+
 val fold_right : (char -> 'a -> 'a) -> string -> 'a -> 'a
 (** [fold_right f s b] is
     [f s.[0] (f s.[1] (... (f s.[n-1] b) ...))]
@@ -378,6 +390,12 @@ val fold_right : (char -> 'a -> 'a) -> string -> 'a -> 'a
     [String.fold_right (fun c a -> if c = ' ' then a+1 else a) "a b c" 0 = 2]
 *)
 
+val fold_righti : (int -> char -> 'a -> 'a) -> string -> 'a -> 'a
+(** As [fold_right], but with the index of the element as additional argument
+
+    @since 2.3.0
+*)
+
 val filter : (char -> bool) -> string -> string
 (** [filter f s] returns a copy of string [s] in which only
     characters [c] such that [f c = true] remain.
@@ -927,10 +945,13 @@ sig
   (** {6 String traversals}*)
 
   val map : (char -> char) -> [>`Read] t -> _ t
+  val mapi : (int -> char -> char) -> [>`Read] t -> _ t
 
   val fold_left : ('a -> char -> 'a) -> 'a -> [> `Read] t -> 'a
+  val fold_lefti : ('a -> int -> char -> 'a) -> 'a -> [> `Read] t -> 'a
 
   val fold_right : (char -> 'a -> 'a) -> [> `Read] t -> 'a -> 'a
+  val fold_righti : (int -> char -> 'a -> 'a) -> [> `Read] t -> 'a -> 'a
 
   val filter : (char -> bool) -> [> `Read] t -> _ t
 
