@@ -283,6 +283,12 @@ val fold_left : ('a -> char -> 'a) -> 'a -> t -> 'a
     tail-recursively, where [sus = (s, i, n)].  Equivalent to
     [List.fold_left f e (explode sus)].  *)
 
+val fold_lefti : ('a -> int -> char -> 'a) -> 'a -> t -> 'a
+(** As [fold_left], but with the index of the element as additional argument
+
+    @since 2.3.0
+*)
+
 val fold_right : (char -> 'a -> 'a) -> t -> 'a -> 'a
 (** [foldr f e sus] folds [f] over [sus] from right to left.  That is,
     evaluates [f s.[i] (f s.[i+1] (f ... (f s.[i+n-1] e) ...))]
@@ -290,6 +296,12 @@ val fold_right : (char -> 'a -> 'a) -> t -> 'a -> 'a
     [List.fold_right f e (explode sus)].
 *)
 
+val fold_righti : (int -> char -> 'a -> 'a) -> t -> 'a -> 'a
+(** As [fold_right], but with the index of the element as additional argument
+
+    @since 2.3.0
+*)
+
 val iter : (char -> unit) -> t -> unit
 (** [iter f sus] applies [f] to all characters of [sus], from left to
     right.  Equivalent to [List.iter f (explode sus)].
