@@ -43,7 +43,7 @@ let splice s1 off len s2 =
   let off  = if off < 0 then len1 + off - 1 else off  in
   let len  = int_min (len1 - off) len                 in
   let out_len = len1 - len + len2                     in
-  let s = String.create out_len in
+  let s = Bytes.create out_len in
   String.blit s1 0 s 0 off; (* s1 before splice point *)
   String.blit s2 0 s off len2; (* s2 at splice point *)
   String.blit s1 (off+len) s (off+len2) (len1 - (off+len)); (* s1 after off+len *)
@@ -53,11 +53,11 @@ type t =
     Empty                             (**An empty rope*)
   | Concat of t * int * t * int * int (**[Concat l ls r rs h] is the concatenation of
                                          ropes [l] and [r], where [ls] is the total
-										 length of [l], [rs] is the length of [r]
-										 and [h] is the height of the node in the
-										 tree, used for rebalancing. *)
+                                         length of [l], [rs] is the length of [r]
+                                         and [h] is the height of the node in the
+                                         tree, used for rebalancing. *)
   | Leaf of int * UTF8.t              (**[Leaf l t] is string [t] with length [l],
-										 measured in number of Unicode characters.*)
+                                         measured in number of Unicode characters.*)
 
 type forest_element = { mutable c : t; mutable len : int }
 
@@ -856,7 +856,7 @@ let split r sep =
   split (of_string "OCaml, the coolest FP language.") (of_char '.') = \
     (of_string "OCaml, the coolest FP language", empty)
   Result.(catch (split (of_string "OCaml, the coolest FP language.")) \
-		(of_char '!') |> is_exn Not_found)
+        (of_char '!') |> is_exn Not_found)
 *)
 
 let rsplit (r:t) sep =
@@ -868,7 +868,7 @@ let rsplit (r:t) sep =
   rsplit (of_string "OCaml, the coolest FP language.") (of_char 'O') = \
     (empty, of_string "Caml, the coolest FP language.")
   Result.(catch (rsplit (of_string "OCaml, the coolest FP language.")) \
-		(of_char '!') |> is_exn Not_found)
+        (of_char '!') |> is_exn Not_found)
 *)
 
 (** An implementation of [nsplit] in one pass.
@@ -878,31 +878,50 @@ let rsplit (r:t) sep =
     avoid a call to [List.rev].  *)
 let nsplit str sep =
   if is_empty str then []
-  else let seplen = length sep in
-    let rec aux acc ofs = match
-      try Some(rfind_from str ofs sep)
-      with Not_found -> None
-    with
-    | Some idx ->
-      (* at this point, [idx] to [idx + seplen - 1] contains the
-			separator, which is useless to us on the other hand,
-			[idx + seplen] to [ofs] contains what's just after the
-			separator, which is what we want*)
-      let end_of_occurrence = idx + seplen in
-      if end_of_occurrence >= ofs then
-        aux acc (idx - 1) (*We may be at the end of the string*)
+  else if is_empty sep then invalid_arg "nsplit: empty sep not allowed"
+  else
+    (* str is not empty *)
+    let seplen = length sep in
+    let rec aux acc ofs =
+      if ofs >= 0 then (
+        match
+          try Some (rfind_from str ofs sep)
+          with Not_found -> None
+        with
+        | Some idx -> (* sep found *)
+          let end_of_sep = idx + seplen - 1 in
+          if end_of_sep = ofs (* sep at end of str *)
+          then aux (empty::acc) (idx - 1)
+          else
+            let token = sub str (end_of_sep + 1) (ofs - end_of_sep) in
+            aux (token::acc) (idx - 1)
+        | None -> (* sep NOT found *)
+          (sub str 0 (ofs + 1))::acc
+      )
       else
-        aux ( sub str end_of_occurrence ( ofs - end_of_occurrence + 1) :: acc ) (idx - 1)
-    | None -> (sub str 0 (ofs + 1))::acc
+        (* Negative ofs: the last sep started at the beginning of str *)
+        empty::acc
     in
     aux [] (length str - 1 )
+
 (*$T nsplit
-  nsplit (of_string "OCaml, the coolest FP language.") (of_char ' ') = \
-    List.map of_string ["OCaml,"; "the"; "coolest"; "FP"; "language."]
-  nsplit (of_string "OCaml, the coolest FP language.") (of_char 'o') = \
-    List.map of_string ["OCaml, the c"; "lest FP language."]
-  nsplit (of_string "OCaml, the coolest FP language.") (of_char '!') = \
-    List.map of_string ["OCaml, the coolest FP language."]
+  nsplit (of_string "OCaml, the coolest FP language.") (of_char 'o') \
+    |> List.map to_string = ["OCaml, the c"; ""; "lest FP language."]
+  nsplit (of_string "OCaml, the coolest FP language.") (of_char '!') \
+    |> List.map to_string = ["OCaml, the coolest FP language."]
+  nsplit (of_string "1,2,3") (of_string ",") \
+    |> List.map to_string = ["1"; "2"; "3"]
+  nsplit (of_string "a;b;c") (of_string ";") \
+    |> List.map to_string = ["a"; "b"; "c"]
+  nsplit (of_string "") (of_string "x") = []
+  try ignore (nsplit (of_string "abc") (of_string "")); false \
+    with Invalid_argument _ -> true
+  nsplit (of_string "a/b/c") (of_string "/") |> List.map to_string \
+    = ["a"; "b"; "c"]
+  nsplit (of_string "/a/b/c//") (of_string "/") |> List.map to_string \
+    = [""; "a"; "b"; "c"; ""; ""]
+  nsplit (of_string "FOOaFOObFOOcFOOFOO") (of_string "FOO") |> List.map to_string \
+    = [""; "a"; "b"; "c"; ""; ""]
 *)
 
 let join = concat
@@ -962,8 +981,8 @@ let read_char i =
   let len = UTF8.length0 (Char.code n0) in
   if len = 1 then UChar.of_char n0
   else
-    let s = String.create len in
-    String.set s 0 n0;
+    let s = Bytes.create len in
+    Bytes.set s 0 n0;
     ignore(really_input i s 1 ( len - 1));
     UTF8.get s 0
 
